@@ -35,8 +35,8 @@ from osgeo import gdal_array
 from shapely.geometry import Polygon
 from shapely.wkt import loads as shply_loads
 from six import PY3
+from cartopy.crs import epsg as ccrs_from_epsg, PlateCarree
 
-# mpl_toolkits.basemap -> imported when GeoArray.show_map() is used
 # dill -> imported when dumping GeoArray
 
 try:
@@ -1128,6 +1128,28 @@ class GeoArray(object):
 
         return image2plot, gt, prj
 
+    @staticmethod
+    def _get_cmap_vmin_vmax(cmap, vmin, vmax, pmin, pmax, image2plot, nodataVal):
+        from matplotlib import pyplot as plt
+        # set color palette
+        palette = plt.get_cmap(cmap) if cmap and isinstance(cmap, str) else cmap if cmap else plt.get_cmap('gray')
+        palette = copy(palette)  # allows to modify the colormap as in-place modifications are not allowed anymore
+        if nodataVal is not None and np.std(image2plot) != 0:  # do not show nodata
+            image2plot = np.ma.masked_equal(image2plot, nodataVal)
+            vmin_auto, vmax_auto = \
+                np.nanpercentile(image2plot.compressed(), pmin), np.nanpercentile(image2plot.compressed(), pmax)
+            palette.set_bad('aqua', 0)
+        else:
+            vmin_auto, vmax_auto = np.nanpercentile(image2plot, pmin), np.nanpercentile(image2plot, pmax)
+
+        vmin = vmin if vmin is not None else vmin_auto
+        vmax = vmax if vmax is not None else vmax_auto
+
+        palette.set_over('1')
+        palette.set_under('0')
+
+        return palette, vmin, vmax
+
     def show(self, xlim=None, ylim=None, band=None, boundsMap=None, boundsMapPrj=None, figsize=None,
              interpolation='none', vmin=None, vmax=None, pmin=2, pmax=98, cmap=None, nodataVal=None,
              res_factor=None, interactive=False, ax=None, ignore_rotation=False):
@@ -1166,27 +1188,12 @@ class GeoArray(object):
             self._get_plottable_image(xlim, ylim, band, boundsMap=boundsMap, boundsMapPrj=boundsMapPrj,
                                       res_factor=res_factor, nodataVal=nodataVal, ignore_rotation=ignore_rotation)
 
-        # set color palette
-        palette = plt.get_cmap(cmap) if cmap and isinstance(cmap, str) else cmap if cmap else plt.get_cmap('gray')
-        palette = copy(palette)  # allows to modify the colormap as in-place modifications are not allowed anymore
-        if nodataVal is not None and np.std(image2plot) != 0:  # do not show nodata
-            image2plot = np.ma.masked_equal(image2plot, nodataVal)
-            vmin_auto, vmax_auto = \
-                np.nanpercentile(image2plot.compressed(), pmin), np.nanpercentile(image2plot.compressed(), pmax)
-            palette.set_bad('aqua', 0)
-        else:
-            vmin_auto, vmax_auto = np.nanpercentile(image2plot, pmin), np.nanpercentile(image2plot, pmax)
-
-        vmin = vmin if vmin is not None else vmin_auto
-        vmax = vmax if vmax is not None else vmax_auto
-
-        palette.set_over('1')
-        palette.set_under('0')
+        palette, vmin, vmax = self._get_cmap_vmin_vmax(cmap, vmin, vmax, pmin, pmax, image2plot, nodataVal)
 
         # check availability of holoviews
-        if not find_loader('holoviews'):
+        if interactive and not find_loader('holoviews'):
             warnings.warn("Interactive mode requires holoviews. Install it by running, e.g., "
-                          "'conda install --yes -c ioam bokeh holoviews'. Using non-interactive mode.")
+                          "'conda install -c conda-forge holoviews'. Using non-interactive mode.")
             interactive = False
 
         if interactive and image2plot.ndim == 3:
@@ -1227,9 +1234,9 @@ class GeoArray(object):
 
             return im
 
-    def show_map(self, xlim=None, ylim=None, band=0, boundsMap=None, boundsMapPrj=None, ax=None, figsize=None,
-                 interpolation='none', vmin=None, vmax=None, cmap=None, nodataVal=None, res_factor=None,
-                 return_map=False, zoomable=False):
+    def show_map(self, xlim=None, ylim=None, band=0, boundsMap=None, boundsMapPrj=None, out_epsg=None, figsize=None,
+                 interpolation='none', vmin=None, vmax=None, pmin=2, pmax=98, cmap=None, nodataVal=None,
+                 res_factor=None, return_map=False):
         """
 
         :param xlim:
@@ -1237,43 +1244,22 @@ class GeoArray(object):
         :param band:            band index (starting with 0)
         :param boundsMap:       xmin, ymin, xmax, ymax
         :param boundsMapPrj:
-        :param ax:              allows to pass a matplotlib axis object where figure is plotted into
+        :param out_epsg:        EPSG code of the output projection
         :param figsize:
         :param interpolation:
-        :param vmin:
-        :param vmax:
+        :param vmin:            darkest pixel value to be included in stretching
+        :param vmax:            brightest pixel value to be included in stretching
+        :param pmin:            percentage to be used for excluding the darkest pixels from stretching (default: 2)
+        :param pmax:            percentage to be used for excluding the brightest pixels from stretching (default: 98)
         :param cmap:
         :param nodataVal:
         :param res_factor:      <float> resolution factor for downsampling of the image to be plotted in order to save
                                 plotting time and memory (default=None -> downsampling is performed to 1000x1000)
         :param return_map:
-        :param zoomable:        <bool> enable or disable zooming via mpld3
         :return:
         """
-
-        if not find_loader('mpl_toolkits.basemap'):
-            raise ImportError('This function requires Basemap. You need to install basemap manually (see www./'
-                              'matplotlib.org/basemap) if you want to plot maps. It is not automatically installed.')
-
-        from mpl_toolkits.basemap import Basemap
         from matplotlib import pyplot as plt
 
-        mpld3_avl = find_loader('mpld3')
-        if not mpld3_avl:
-            warnings.warn('mpld3 is not available. Zooming disabled.')
-            zoomable = False
-
-        if zoomable:
-            import mpld3
-            mpld3.enable_notebook()
-        elif mpld3_avl:
-            import mpld3
-            # noinspection PyBroadException
-            try:
-                mpld3.disable_notebook()
-            except Exception:
-                pass
-
         assert self.geotransform and tuple(self.geotransform) != (0, 1, 0, 0, 0, -1), \
             'A valid geotransform is needed for a map visualization. Got %s.' % list(self.geotransform)
         assert self.projection, "A projection is needed for a map visualization. Got '%s'." % self.projection
@@ -1282,126 +1268,38 @@ class GeoArray(object):
         nodataVal = nodataVal if nodataVal is not None else self.nodata
         image2plot, gt, prj = self._get_plottable_image(xlim, ylim, band, boundsMap=boundsMap,
                                                         boundsMapPrj=boundsMapPrj, res_factor=res_factor,
-                                                        nodataVal=nodataVal, out_prj='epsg:4326')
-
-        # calculate corner coordinates of plot
-        # if boundsMap:
-        #    boundsMapPrj = boundsMapPrj if boundsMapPrj else self.prj
-        #    if not prj_equal(boundsMapPrj, 4326):
-        #        boundsMap = reproject_shapelyGeometry(box(*boundsMap), boundsMapPrj, 4626).bounds
-        #    xmin, ymin, xmax, ymax = boundsMap
-        #    UL_XY, UR_XY, LR_XY, LL_XY = (xmin,ymax), (xmax, ymax), (xmax,ymin), (xmin, ymin)
-        # else:
-        UL_XY, UR_XY, LR_XY, LL_XY = [(YX[1], YX[0]) for YX in GeoArray(image2plot, gt, prj).box.boxMapYX]
-        center_lon, center_lat = (UL_XY[0] + UR_XY[0]) / 2., (UL_XY[1] + LL_XY[1]) / 2.
+                                                        nodataVal=nodataVal)
 
         # create map
+        def get_cartopy_crs_from_epsg(epsg_code):
+            try:
+                return ccrs_from_epsg(epsg_code)
+            except ValueError:
+                if epsg_code == 4326:
+                    return PlateCarree()
+                else:
+                    raise NotImplementedError('The show_map() method currently does not support the given projection.')
+
+        crs_in = get_cartopy_crs_from_epsg(self.epsg)
+        crs_out = get_cartopy_crs_from_epsg(out_epsg if out_epsg is not None else self.epsg)
+
         fig = plt.figure(figsize=figsize)
-        plt.subplots_adjust(left=0.05, right=0.95, top=0.90, bottom=0.05, wspace=0.15, hspace=0.05)
-        ax = ax if ax is not None else plt.subplot(111)
+        ax = plt.axes(projection=crs_out)
 
-        m = Basemap(projection='tmerc', ellps='WGS84', resolution=None, lon_0=center_lon, lat_0=center_lat,
-                    urcrnrlon=UR_XY[0], urcrnrlat=UR_XY[1], llcrnrlon=LL_XY[0], llcrnrlat=LL_XY[1])
+        ax.set_extent(self.box.boundsMap, crs=crs_in)
 
-        # set color palette
-        palette = cmap if cmap else plt.get_cmap('gray')
+        palette, vmin, vmax = self._get_cmap_vmin_vmax(cmap, vmin, vmax, pmin, pmax, image2plot, nodataVal)
         if nodataVal is not None and np.std(image2plot) != 0:  # do not show nodata
             image2plot = np.ma.masked_equal(image2plot, nodataVal)
-            vmin_auto, vmax_auto = \
-                np.nanpercentile(image2plot.compressed(), 2), np.nanpercentile(image2plot.compressed(), 98)
-            palette.set_bad('aqua', 0)
-        else:
-            vmin_auto, vmax_auto = np.nanpercentile(image2plot, 2), np.nanpercentile(image2plot, 98)
-        vmin = vmin if vmin is not None else vmin_auto
-        vmax = vmax if vmax is not None else vmax_auto
-        palette.set_over('1')
-        palette.set_under('0')
+        ax.imshow(image2plot, cmap=palette, interpolation=interpolation, vmin=vmin, vmax=vmax,
+                  origin='upper', transform=crs_in, extent=list(self.box.boundsMap))
 
-        # add image to map (y-axis must be inverted for basemap)
-        if zoomable:
-            m.imshow(image2plot, palette, interpolation=interpolation, vmin=vmin, vmax=vmax)
-        else:
-            m.imshow(np.flipud(image2plot), palette, interpolation=interpolation, vmin=vmin, vmax=vmax)
-
-        # add coordinate grid lines
-        parallels = np.arange(-90, 90., 0.25)  # TODO make this adjustable
-        # parallels = np.arange(-90, 90., 0.1)
-        m.drawparallels(parallels, labels=[1, 0, 0, 0], fontsize=12, linewidth=0.4)
-
-        meridians = np.arange(-180., 180., 0.25)
-        # meridians = np.arange(-180., 180., 0.1)
-        m.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=12, linewidth=0.4)
+        # show grid lines
+        # NOTE: the current version of cartopy can only label PlateCarree grid lines - (crs=crs) is not yet supported
+        ax.gridlines(draw_labels=True, linewidth=2, color='gray', alpha=0.5, linestyle='--')
 
         if return_map:
-            return fig, ax, m
-        else:
-            plt.show()
-
-    def show_map_utm(self, xlim=None, ylim=None, band=0, figsize=None, interpolation='none', cmap=None,
-                     nodataVal=None, vmin=None, vmax=None, res_factor=None, return_map=False):
-
-        if not find_loader('mpl_toolkits.basemap'):
-            raise ImportError('This function requires Basemap. You need to install basemap manually (see www./'
-                              'matplotlib.org/basemap) if you want to plot maps. It is not automatically installed.')
-
-        from mpl_toolkits.basemap import Basemap
-        from matplotlib import pyplot as plt
-
-        warnings.warn(UserWarning('This function is still under construction and may not work as expected!'))
-        # TODO debug this function
-
-        # get image to plot
-        nodataVal = nodataVal if nodataVal is not None else self.nodata
-        image2plot, gt, prj = self._get_plottable_image(xlim, ylim, band, res_factor, nodataVal)
-
-        # calculate corner coordinates of plot
-        box2plot = GeoArray(image2plot, gt, prj).box
-        # UL_XY, UR_XY, LR_XY, LL_XY = [(YX[1], YX[0]) for YX in GeoArray(image2plot, gt, prj).box.boxMapYX]
-        # Xarr, Yarr = self.box.get_coordArray_MapXY(prj=EPSG2WKT(4326))
-        UL_XY, UR_XY, LR_XY, LL_XY = [transform_any_prj(self.projection, 'epsg:4326', x, y) for y, x in
-                                      box2plot.boxMapYX]
-        center_X, center_Y = (UL_XY[0] + UR_XY[0]) / 2., (UL_XY[1] + LL_XY[1]) / 2.
-        center_lon, center_lat = transform_any_prj(prj, 'epsg:4326', center_X, center_Y)
-        print(center_lon, center_lat)
-
-        # create map
-        fig = plt.figure(figsize=figsize)
-        plt.subplots_adjust(left=0.05, right=0.95, top=0.90, bottom=0.05, wspace=0.15, hspace=0.05)
-        ax = plt.subplot(111)
-        print(UL_XY, UR_XY, LR_XY, LL_XY)
-        #        m = Basemap(projection='tmerc', resolution=None, lon_0=center_lon, lat_0=center_lat,
-        #                    urcrnrx=UR_XY[0], urcrnry=UR_XY[1], llcrnrx=LL_XY[0], llcrnry=LL_XY[1])
-        m = Basemap(projection='tmerc', ellps='WGS84', resolution=None, lon_0=center_lon, lat_0=center_lat,
-                    urcrnrlon=UR_XY[0], urcrnrlat=UR_XY[1], llcrnrlon=LL_XY[0], llcrnrlat=LL_XY[1],
-                    k_0=0.9996, rsphere=(6378137.00, 6356752.314245179), suppress_ticks=False)
-        # m.pcolormesh(Xarr, Yarr, self[:], cmap=plt.cm.jet)
-
-        # set color palette
-        palette = cmap if cmap else plt.get_cmap('gray')
-        if nodataVal is not None:  # do not show nodata
-            image2plot = np.ma.masked_equal(image2plot, nodataVal)
-            vmin_auto, vmax_auto = \
-                np.nanpercentile(image2plot.compressed(), 2), np.nanpercentile(image2plot.compressed(), 98)
-            palette.set_bad('aqua', 0)
-        else:
-            vmin_auto, vmax_auto = np.nanpercentile(image2plot, 2), np.nanpercentile(image2plot, 98)
-        vmin = vmin if vmin is not None else vmin_auto
-        vmax = vmax if vmax is not None else vmax_auto
-        palette.set_over('1')
-        palette.set_under('0')
-
-        # add image to map (y-axis must be inverted for basemap)
-        m.imshow(np.flipud(image2plot), palette, interpolation=interpolation, vmin=vmin, vmax=vmax)
-
-        # add coordinate grid lines
-        parallels = np.arange(-90, 90., 0.25)
-        m.drawparallels(parallels, labels=[1, 0, 0, 0], fontsize=12, linewidth=0.4)
-
-        meridians = np.arange(-180., 180., 0.25)
-        m.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=12, linewidth=0.4)
-
-        if return_map:
-            return fig, ax, m
+            return fig, ax
         else:
             plt.show()
 
